@@ -242,8 +242,64 @@ class Client(object):
     def get_quotes_pdf(self):
         raise NotImplementedError
 
-    def get_relationships(self):
-        raise NotImplementedError
+    def get_relationships(
+        self,
+        module_name,
+        module_id,
+        link_field_name,
+        related_module_query,
+        related_fields,
+        related_module_link_name_to_fields_array,
+        deleted=False,
+        order_by="",
+        offset=0,
+        limit=False,
+    ):
+        """Retrieve a collection of beans that are related to the specified
+        bean and optionally return relationship data for those related beans.
+
+        Args:
+            module_name: The name of the module that the primary record is
+                from. This name should be the name the module was developed
+                under (changing a tab name is studio does not affect the name
+                that should be passed into this method)..
+            module_id: The ID of the bean in the specified module
+            link_field_name: The name of the link field to return records from.
+                This name should be the name the relationship.
+            related_module_query: A portion of the where clause of the SQL
+                statement to find the related items.  The SQL query will
+                already be filtered to only include the beans that are related
+                to the specified bean.
+            related_fields - Array of related bean fields to be returned.
+            related_module_link_name_to_fields_array - For every related bean
+                returrned, specify link fields name to fields info for that
+                bean to be returned. For ex.'link_name_to_fields_array' =>
+                array(array('name' =>  'email_addresses', 'value' =>
+                array('id', 'email_address', 'opt_out', 'primary_address'))).
+            deleted: false if deleted records should not be include, true if
+                deleted records should be included.
+            order_by: field to order the result sets by
+            offset: where to start in the return
+            limit: number of results to return (defaults to all)
+
+        Returns:
+            A dict.
+        """
+
+        data = [
+            self.session_id,
+            module_name,
+            module_id,
+            link_field_name,
+            related_module_query,
+            related_fields,
+            related_module_link_name_to_fields_array,
+            int(deleted),
+            order_by,
+            offset,
+            limit,
+        ]
+        return self._post("get_relationships", data)
 
     def get_report_entries(self):
         raise NotImplementedError
@@ -333,11 +389,85 @@ class Client(object):
     def set_document_revision(self):
         raise NotImplementedError
 
-    def set_note_attachment(self):
-        raise NotImplementedError
+    def set_note_attachment(
+        self,
+        noteid,
+        filename,
+        filecontent,
+        related_module_id=None,
+        related_module_name=None,
+    ):
+        """Add or replace the attachment on a Note. Optionally you can set the
+        relationship of this note to Accounts/Contacts and so on by setting
+        related_module_id, related_module_name
 
-    def set_relationship(self):
-        raise NotImplementedError
+        Args:
+            noteid: The ID of the Note containing the attachment
+            filename: The file name of the attachment
+            filecontent: The binary contents of the file.
+            related_module_id: module id to which this note to related to
+            related_module_name: module name to which this note to related to
+
+        Returns:
+            A dict.
+        """
+        data = [
+            self.session_id,
+            {
+                "id": noteid,
+                "filename": filename,
+                "file": filecontent,
+                "related_module_id": related_module_id,
+                "related_module_name": related_module_name,
+            },
+        ]
+
+        return self._post("set_note_attachment", data)
+
+
+    @valid_parameters
+    def set_relationship(
+        self,
+        module_name,
+        module_id,
+        link_field_name,
+        related_ids,
+        name_value_list=[],
+        delete=0,
+    ):
+        """Set a single relationship between two beans. The items are related
+        by module name and id.
+
+        Args:
+            module_name: name of the module that the primary record is from.
+                This name should be the name the module was developed under
+                (changing a tab name is studio does not affect the name that
+                should be passed into this method)..
+            module_id: The ID of the bean in the specified module_name
+            link_field_name: name of the link field which relates to the other
+                module for which the relationship needs to be generated.
+            related_ids: array of related record ids for which relationships
+                needs to be generated
+            name_value_list: The keys of the array are the SugarBean
+                attributes, the values of the array are the values the
+                attributes should have.
+            delete: Optional, if the value 0 or nothing is passed then it will
+                add the relationship for related_ids and if 1 is passed, it
+                will delete this relationship for related_ids
+
+        Returns:
+            A dict.
+        """
+        data = [
+            self.session_id,
+            module_name,
+            module_id,
+            link_field_name,
+            related_ids,
+            name_value_list,
+            delete,
+        ]
+        return self._post("set_relationship", data)
 
     def set_relationships(self):
         raise NotImplementedError
